import re

# 7-bit C1 ANSI escape sequences
# https://www.cnblogs.com/longdouhzt/archive/2012/09/25/2703173.html
ANSI_ESCAPE = re.compile(r'\x1B[@-_][0-?]*[ -/]*[@-~]')


class BaseString():

    @classmethod
    def r_strip_one_char(cls, string):
        """
        去除右边的一个字符
        :param string: 
        :return: 
        """
        string = string[:-1]
        return string

    @classmethod
    def r_strip_substring(cls, string, substring):
        """
        移除子字符串
        :param string:
        :param substring:
        :return:
        """
        return string.rstrip(substring)

    @classmethod
    def r_strip_char(cls, string, number):
        """
        删除右边指定个数的字符串
        :param string: 需要处理的字符串
        :param number: 右边需要删除的字符个数
        :return: 
        """
        if number > 0:
            number = 0 - number
        elif number == 0:
            return string

        string = string[:number]
        return string

    @classmethod
    def get_strings_line_num(cls, strings, substrings, start=None, end=None):
        """
        方法用于统计字符串里某个字符出现的次数
        sub -- 搜索的子字符串
        start -- 字符串开始搜索的位置。默认为第一个字符,第一个字符索引值为0。
        end -- 字符串中结束搜索的位置。字符中第一个字符的索引为 0。默认为字符串的最后一个位置。
        :param filePath:
        :return:
        """
        if start is None:
            start = 0
        if end is None:
            end = len(strings)
        return strings.count(substrings, start=start, end=end)

    def filter_string_blank_space(self, filterobj):
        """
        过滤字符串空格
        例如  list = [' n 1 2', ' c 4s r   ']
        :return ['n12', 'c4sr']

        """
        if isinstance(filterobj, str):
            return "".join(filterobj.split())
        if isinstance(filterobj, list):
            listobj = ["".join(x.split()) for x in filterobj]
            return listobj
        else:
            assert "not have this Type，please check parameter Type" + __file__

    def string_to_list(self, string):
        """
        string 转换成list
        :param string:
        :return:  一个list每个元素一个char
        """
        mlist = [string[x] for x in range(len(string))]
        return mlist

    def parallelIterations(self, lista, listb):
        """
        并行迭代 会迭代出最少的那个  如下面的f被丢弃
            if (lista is None and listb is None):
            lista = ['1', '2', '3', '4', '5']
            listb = ['a', 'b', 'c', 'd', 'e', 'f']
        :param lista:
        :param listb:
        :return:
        """
        for aa, bb in zip(lista, listb):
            yield aa, bb

    def list_to_string(self, mlist):
        """
         b = ['a', 'b', 'c', 'd', 'e', 'f']
        :param mlist:
        :return:  string abcdef
        """
        return "".join(mlist)

    def find_sub(self, strings, substr):
        """
        strings是否存在sunstr
        :param strings:
        :param substr:
        :return:
        """
        num = strings.find(substr)
        if num == -1:
            return False
        else:
            return num

    @classmethod
    def cleanSemicolon(cls, text):
        """
        中文分号到英文分号的替换
        字符串前后分号的去除
        连续分号的合并
        分号前后空格的去除
        :param text:
        :return:
        """
        text = text.replace('；', ';')
        text = re.sub("\s+;", ";", text)
        text = re.sub(";\s+", ";", text)
        text = re.sub(";+", ";", text)
        text = re.sub("^;", "", text)
        text = re.sub(";$", "", text)
        return text.strip()

    def intercept_strings(self, str):
        """
        str = ‘0123456789’
        print str[0:3] #截取第一位到第三位的字符
        print str[:] #截取字符串的全部字符
        print str[6:] #截取第七个字符到结尾
        print str[:-3] #截取从头开始到倒数第三个字符之前
        print str[2] #截取第三个字符
        print str[-1] #截取倒数第一个字符
        print str[::-1] #创造一个与原字符串顺序相反的字符串
        print str[-3:-1] #截取倒数第三位与倒数第一位之前的字符
        print str[-3:] #截取倒数第三位到结尾
        print str[:-5:-3] #逆序截取，具体啥意思没搞明白？
        截取字符串
        :param str: 仅供参考使用
        :return:
        """

    def eval(self, strs):
        """
        将str当有效表达式
        :return:
        """
        return eval(strs)

    def lower(self, strs):
        """
        小写
        :param strs:
        :return:
        """
        return strs.lower()

    def upper(self, strs):
        """
        大写
        :param strs:
        :return:
        """
        return strs.upper()

    def swapcase(self, strs):
        """
        将字符串中的大写字母转为小写，小写字母转为大写
        :param strs:
        :return:
        """
        return strs.swapcase()

    def capitalize(self, strs):
        """
        字符串中的第一个字母大写，其他小写
        :param strs:
        :return:
        """
        return strs.capitalize()

    def title(self, strs):
        """
        将字符串中的每隔单词的首字母大写
        :param strs:
        :return:
        """
        return strs.title()

    def fillchar(self):
        """
        返回一个指定长度width的字符串str这个字符串在中间，其他位置用fillchar补全，默认是空格
        str3 = "1234"

        print(str3.center(15, "*"))
        print(str3.ljust(15, "*"))
        print(str3.rjust(15, "*"))
        print(str3.zfill(15)) #0补充

        ******1234*****
        1234***********
        ***********1234
        000000000001234


        s为字符串
        s.isalnum() 所有字符都是数字或者字母
        s.isalpha() 所有字符都是字母
        s.isdigit() 所有字符都是数字
        s.islower() 所有字符都是小写
        s.isupper() 所有字符都是大写
        s.istitle() 所有单词都是首字母大写，像标题
        s.isspace() 所有字符都是空白字符、\t、\n、\r
        参考
        :return:
        """
        pass

    @classmethod
    def hasnum(self, strings):
        """
        判断字符串是否有数字
        :param strings:
        :return:
        """
        return any(char.isdigit() for char in strings)

    @classmethod
    def chinese2digits(cls, uchars_chinese):

        common_used_numerals_tmp = {'零': 0, '一': 1, '二': 2, '两': 2, '三': 3, '四': 4, '五': 5, '六': 6, '七': 7, '八': 8,
                                    '九': 9,
                                    '十': 10, '百': 100, '千': 1000, '万': 10000, '亿': 100000000}
        total = 0
        r = 1  # 表示单位：个十百千...
        for i in range(len(uchars_chinese) - 1, -1, -1):
            # print(uchars_chinese[i])
            val = common_used_numerals_tmp.get(uchars_chinese[i])
            if val >= 10 and i == 0:  # 应对 十三 十四 十*之类
                if val > r:
                    r = val
                    total = total + val
                else:
                    r = r * val
                    # total =total + r * x
            elif val >= 10:
                if val > r:
                    r = val
                else:
                    r = r * val
            else:
                total = total + r * val
        return total

    @classmethod
    def is_space(cls, strings):
        """
        判断字符串是否全是空格
        :param strings:
        :return:
        """
        return strings.isspace()

    def str_full_to_half(self, ustring):
        """
        把字符串全角转半角
        全角字符unicode编码从65281~65374 （十六进制 0xFF01 ~ 0xFF5E）
        半角字符unicode编码从33~126 （十六进制 0x21~ 0x7E）
        空格比较特殊,全角为 12288（0x3000）,半角为 32 （0x20）
        而且除空格外,全角/半角按unicode编码排序在顺序上是对应的
        所以可以直接通过用+-法来处理非空格数据,对空格单独处理
        """
        rstring = ""
        for uchar in ustring:
            # 返回值是对应的十进制整数。
            # ord() 函数是 chr() 函数（对于8位的ASCII字符串）或 unichr() 函数（对于Unicode对象）的配对函数，它以一个字符（长度为1的字符串）作为参数，返回对应的 ASCII 数值，或者 Unicode 数值，如果所给的 Unicode 字符超出了你的 Python 定义范围，则会引发一个 TypeError 的异常。
            inside_code = ord(uchar)
            if inside_code == 0x3000:
                inside_code = 0x0020
            else:
                inside_code -= 0xfee0
            if not (0x0020 < inside_code and inside_code <= 0x7e):  # 转完之后不是半角字符返回原来的字符
                rstring += uchar
                continue
            rstring += chr(inside_code)
        return rstring

    def str_half_to_full(self, ustring):
        """把字符串半角转全角"""
        rstring = ""
        for uchar in ustring:
            inside_code = ord(uchar)
            if inside_code == 0x0020:  # 除了空格其他的全角半角的公式为:半角=全角-0xfee0
                inside_code = 0x3000
            else:
                if not (0x0020 < inside_code and inside_code <= 0x7e):
                    rstring += uchar
                    continue
                inside_code += 0xfee0
                rstring += chr(inside_code)
        return rstring

    def is_chinese(self, strings):
        """
        检查整个字符串是否包含中文
        :param string: 需要检查的字符串
        :return: bool
        """
        for ch in strings:
            if u'\u4e00' <= ch <= u'\u9fff':
                return True
        return False

    def delete_char(self, strings):
        """
        去除特殊字符，只保留汉子，字母、数字
        :return:
        """
        sub_str = re.sub("([^\u4e00-\u9fa5\u0030-\u0039\u0041-\u005a\u0061-\u007a])", "", strings)
        return sub_str

    def is_sub_str(self, strs, substrs):
        """
        判断substrs 是否存在strs中
        strs.find(substrs)>=0
        strs.index(substrs)>=0
        strs.count(substrs)>=0
        :param strs:
        :param sbustrs:
        :return:
        """
        return substrs in strs

    def remove_escape_codes(self, text):
        """
        Remove ANSI escapes codes from string. It's used to remove
        "colors" from log messages.
        """
        return ANSI_ESCAPE.sub("", text)



import csv


class BaseCsv(object):

    def __init__(self):
        pass

    def read_csv(self, filepath):
        """
        根据文件路径逐行读取csv文件
        :param filepath:csv文件路径
        :return:csv每行记录
        """
        # mac_roman 原因
        # https://stackoverflow.com/questions/21504319/python-3-csv-file-giving-unicodedecodeerror-utf-8-codec-cant-decode-byte-err
        with open(filepath, "r",encoding='mac_roman') as f:
            reader = csv.reader(f)
            for row in reader:
                yield row

    def read_all_csv(self, filepath):
        """
        根据文件路径读取csv文件所有行
        :param filepath:csv文件路径
        :return:csv所有行记录
        """
        with open(filepath, "r") as f:
            reader = csv.reader(f)
            result = list(reader)
            return result




import json
###########################################
# 同项目调用基础包
import os
import sys
import time

import requests

from re_common.baselibrary.mthread.MThreadingRun import MThreadingRun
from re_common.baselibrary.mthread.mythreading import ThreadInfo, ThreadVal, ThreadPoolManger

filepath = os.path.abspath(__file__)
pathlist = filepath.split(os.sep)
pathlist = pathlist[:-4]
TopPath = os.sep.join(pathlist)
sys.path.insert(0, TopPath)
print(TopPath)
############################################

from re_common.baselibrary.utils.basedir import BaseDir
from re_common.baselibrary.utils.basefile import BaseFile
from re_common.baselibrary.utils.baserequest import BaseRequest
from re_common.facade.lazy_import import get_streamlogger
from re_common.facade.mysqlfacade import MysqlUtiles


class Kproxy(object):
    def __init__(self):
        self.cur_path = BaseDir.get_file_dir_absolute(__file__)
        self.configfile = BaseFile.get_new_path(self.cur_path, "db.ini")
        self.logger = get_streamlogger()
        self.mysqlutils = MysqlUtiles(self.configfile, "allproxy", self.logger)
        self.bsrequest = BaseRequest()
        self.starttime = time.time()
        self.starttime_val = time.time()

    def get_taiyang_proxy(self, num=6):
        """
        获取太阳代理 每分钟3个
        :param num:
        :return:
        """
        self.starttime = time.time()
        url = "http://http.tiqu.qingjuhe.cn/getip?num={}&type=2&pack=56912&port=1&ts=1&lb=1&pb=45&regions=".format(num)
        BoolResult, errString, r = self.bsrequest.base_request(url,
                                                               timeout=30
                                                               )
        if BoolResult:
            dicts = json.loads(r.text)
            for item in dicts["data"]:
                proxy = item["ip"] + ":" + item["port"]
                sources = "taiyang"
                expire_time = item["expire_time"]
                sql = "insert into proxyall (proxy,sources,expire_time) values ('%s','%s','%s') on DUPLICATE key update stat=1,expire_time='%s'" % (
                    proxy, sources, expire_time, expire_time)
                self.mysqlutils.ExeSqlToDB(sql)
        else:
            self.logger.error("获取失败")

    def val(self, proxy, sources,threadval):
        # 请求地址
        targetUrl = "https://www.baidu.com"
        proxies = {
            "http": "http://%s" % proxy,
            "https": "http://%s" % proxy
        }
        resp = requests.get(targetUrl, proxies=proxies, timeout=5)
        if resp.status_code == 200:
            print(resp.status_code)
            sql = "update proxyall set stat=1 where proxy='%s' and sources='%s';" % (proxy, sources)
            threadval.get_result_queue().put(sql)
            return True
        else:
            sql = "update proxyall set stat=0 where proxy='%s' and sources='%s';" % (proxy, sources)
            # self.mysqlutils.ExeSqlToDB(sql)
            threadval.get_result_queue().put(sql)
            return False

    def val_all(self):
        self.starttime_val = time.time()
        sql = "select proxy,sources from proxyall where stat=1 ORDER BY `update_time`"
        bools, rows = self.mysqlutils.SelectFromDB(sql)
        for row in rows:
            try:
                self.val(row[0], row[1])
            except:
                sql = "update proxyall set stat=0 where proxy='%s' and sources='%s';" % (row[0], row[1])
                self.mysqlutils.ExeSqlToDB(sql)

    def run(self):
        while True:
            start_time = time.time()
            self.get_taiyang_proxy()
            self.val_all()
            use_time = int(time.time() - start_time)
            sleep_time = 100 - use_time
            print("time sleep {}".format(str(sleep_time)))
            if sleep_time >= 3:
                time.sleep(sleep_time)


class DetailThreadRun(MThreadingRun):
    def __init__(self, num):
        self.down = Kproxy()
        super(DetailThreadRun, self).__init__(num)

    @ThreadPoolManger.thread_lock
    def getTask(self, *args, **kwargs):
        sql = "select proxy,sources from proxyall where stat=0 ORDER BY `expire_time` DESC limit 1000"
        bools, rows = self.down.mysqlutils.SelectFromDB(sql)
        return rows

    @ThreadPoolManger.thread_lock
    def getTask2(self, *args, **kwargs):
        sql = "select proxy,sources from proxyall where stat=1 ORDER BY `expire_time` DESC limit 1000"
        bools, rows = self.down.mysqlutils.SelectFromDB(sql)
        return rows

    def setTask(self, results=None, *args, **kwargs):
        if not results:
            return self.BREAK
        for row in results:
            self.add_job(self.func, row[0], row[1])
        rows = self.getTask2()
        for row in rows:
            self.add_job(self.func, row[0], row[1])
        time.sleep(60*2)
        return self.BREAK

    @ThreadPoolManger.thread_lock
    def dealresult(self, *args, **kwargs):
        # for sql in self.results:
        #     self.down.mysqlutils.ExeSqlToDB(sql)
        self.down.mysqlutils.ExeSqlListToDB(self.results)

    def setProxy(self, proxysList=None):
        time.sleep(300)

    def is_break(self):
        return False

    def thread_pool_hook(self, threadinfo: ThreadInfo):
        # 设置代理线程不重启，默认会重启
        return {}

    def fun(self, threadval: ThreadVal, *args, **kwargs):
        proxy,sources = args[0],args[1]
        try:
            self.down.val(proxy, sources, threadval)
        except:
            sql = "update proxyall set stat=0 where proxy='%s' and sources='%s';" % (proxy, sources)
            # self.mysqlutils.ExeSqlToDB(sql)
            threadval.get_result_queue().put(sql)


if __name__ == '__main__':
    down = DetailThreadRun(30)
    down.run()



def import_fun(obj, mod, m=[], modename=""):
    """
    这个方法只能在类里面设置属性
    :param obj:
    :param mod:
    :param m:
    :return:
    """
    if m:
        for item in m:
            module = getattr(mod, item)
            setattr(obj, item, module)
    elif modename:
        setattr(obj, modename, mod)
    else:
        raise ValueError("m 和　modename必须存在一个")


def import_to_val(mod, m=[], globals=None, locals=None):
    """
     设置到变量,在你调用的文件中使用
    :param mod: 一个模块对象
    :param m: 一个列表　mod 里面的变量类和函数
    :param globals: 传入　globals()
    :param locals: 传入　locals()
    :return:
    """
    dicts = {}
    for item in m:
        # module = getattr(mod, item)
        exec("{}=getattr(mod, '{}')".format(item, item))
        dicts[item] = eval(item)
        if globals:
            globals.update({item: eval(item)})
        if locals:
            locals.update({item: eval(item)})
    return dicts



import encodings
import os
import pkgutil


class BaseEncoding(object):

    def __init__(self):
        pass

    def all_encodings(self):
        """

        :return:
        """
        modnames = set([modname for importer, modname, ispkg in pkgutil.walk_packages(
            path=[os.path.dirname(encodings.__file__)], prefix='')])
        aliases = set(encodings.aliases.aliases.values())
        return modnames.union(aliases)

    def is_encoding(self,text=b'\x96'):
        for enc in self.all_encodings():
            try:
                msg = text.decode(enc)
            except Exception:
                continue
            print('Decoding {t} with {enc} is {m}'.format(t=text, enc=enc, m=msg))

# BaseEncoding().is_encoding(b'\0\x93')


onepath = r"F:\db3\ids\base_obj_meta_a.wf_qk.20201101_00000.db3"

import sqlite3


def dataMerge(inpath, attachpath):
    """
    合并两个db3
    :param inpath:
    :param attachpath:
    :return:
    """
    conn = sqlite3.connect(inpath)
    conn.text_factory = str
    cur = conn.cursor()
    attach = 'attach database "' + attachpath + '" as w;'
    sql1 = 'insert into base_obj_meta_a select * from w.base_obj_meta_a;'
    cur.execute(attach)
    cur.execute(sql1)
    conn.commit()
    cur.close()
    conn.close()


for i in range(0, 10):
    attachpath = r"F:\db3\ids\base_obj_meta_a.wf_qk.20201101_00%s.db3" % (str(i))
    dataMerge(onepath, attachpath)
    print(attachpath)

# attachpath = r"\\192.168.31.177\down_data\cnkiwanfang\zt_wf_qk_20200323\db3\zt_wf_qk_20200323_0002.db3"
# dataMerge(onepath, attachpath)
# print(attachpath)





import time

from re_common.baselibrary.utils.basemotor import BaseMotor
from re_common.baselibrary.utils.basepymongo import BasePyMongo


class MongoTools(object):
    def __init__(self, conn_str, db_name, col):
        self.conn_str = conn_str
        self.db_name = db_name
        self.col = col
        self.bs_conn = None

    def conn_mongo_base(self):
        self.bs_conn = BasePyMongo(self.conn_str)
        self.bs_conn.use_db(self.db_name)
        self.bs_conn.create_col(self.col)

    def conn_mongo_motor(self):
        self.bs_conn = BaseMotor()
        self.bs_conn.AsyncIOMotorClient(self.conn_str, self.db_name)
        self.bs_conn.get_col(self.col)

    def get_first_id(self, query):
        for item in self.bs_conn.find({"sub_db_id": "00075", "_id": {"$lt": ""}},
                                      {"_id": 1}).skip(0).limit(1):
            return item["_id"]

    def get_ids(self, ids, query):
        count = 0
        while True:
            lists = []
            query_temp = query.update({"_id": {"$gt": ids}})
            for i in self.bs_conn.find(query_temp, {"_id": 1}).sort([("_id", 1)]).limit(1000000):
                count = count + 1
                ids = i["_id"]
                lists.append((i["_id"]))
                if len(lists) % 10000 == 1:
                    print(len(lists))


if __name__ == "__main__":
    # mt = MongoTools(
    #     conn_str="mongodb://datahouse:vipdatacenter@192.168.31.243:32920/data_warehouse.base_obj_meta_a?authSource=data_warehouse",
    #     db_name="data_warehouse", col="base_obj_meta_a")
    mt = MongoTools(
        conn_str="mongodb://datahouse:vipdatacenter@192.168.31.208:32920,192.168.31.206:32920,192.168.31.243:32920/data_warehouse.base_obj_meta_a?authSource=data_warehouse",
        db_name="data_warehouse", col="base_obj_meta_a")
    mt.conn_mongo_base()
    print(mt.get_first_id({"sub_db_id": "00075"}))



from tempfile import NamedTemporaryFile

from openpyxl import Workbook, load_workbook
from openpyxl.styles import Font, Alignment

"""
openpyxl不支持旧的.xls文件格式，请使用xlrd读取该文件，或将其转换为最新的.xlsx文件格式。
"""


class OpenpyxlUtils(object):
    def __init__(self):
        self.wb = Workbook()  # 该参数guess_types=True 启用了类容和格式推断但为什么Workbook 没有这个参数
        self.ws = None

    def xls_to_xlsx(self, xlsPath):
        """
        https://blog.csdn.net/test_sir_cao/article/details/79133617
        mime  http://www.w3school.com.cn/media/media_mimeref.asp
        python -m pip install pypiwin32

        '类 Workbooks 的 Open 方法无效
        https://blog.csdn.net/xiaozhuzhu1/article/details/48493529
        出现这个错误的原因是，已有的Excel文件中有无法识别的内容，手动打开的时候，会提示：Excel发现不可读取内容，只要确认打开，再保存一下，再用程序打开的时候就不会出现这个问题了。
        有以下几种：
        51 = xlOpenXMLWorkbook (without macro's in 2007-2016, xlsx) 保存为xlsx
        52 = xlOpenXMLWorkbookMacroEnabled (with or without macro's in 2007-2016, xlsm)
        保存为xlsm带宏的格式
        50 = xlExcel12 (Excel Binary Workbook in 2007-2016 with or without macro's, xlsb)
        以二进制保存的工作表
        56 = xlExcel8 (97-2003 format in Excel 2007-2016, xls)
        以xls，2003格式保存的工作表
        :param xlsPath:
        :return:
        """
        import win32com.client
        excel = win32com.client.gencache.EnsureDispatch('Excel.Application')  # 要看MIME手册
        wb = excel.Workbooks.Open(xlsPath)
        wb.SaveAs(xlsPath + 'x', FileFormat=51)
        wb.Close()
        excel.Application.Quit()

    def set_wb_only_read(self, filename, read_only=False):
        """
        只读方式打开excel
        :param filename:
        :param read_only:
        :return:
        """
        self.wb = load_workbook(filename=filename, read_only=read_only)
        return self.wb

    def set_wb_only_write(self):
        """
        按照写的方式打开一个新的wb
        :return:
        """
        self.wb = Workbook(write_only=True)
        return self.wb

    def create_ws(self, sheetname='', sheetnum=0):
        """
        获取 ws sheet 对象 此函数使用_active_sheet_index属性，默认设置为0。
       除非您修改其值，否则您将始终使用此方法获取第一个工作表。
        :return:
        """
        if sheetname:
            self.ws = self.wb.create_sheet(sheetname, sheetnum)
        else:
            # 获取第一个 sheet
            self.ws = self.wb.active
        return self.ws

    def set_ws_title(self, title):
        """
        设置工作表的名字
        :param title:
        :return:
        """
        self.ws.title = title

    def get_ws(self, title):
        """
        获取一个sheet 通过title
        :param title:
        :return:
        """
        self.ws = self.wb[title]
        return self.ws

    def get_list_sheet(self):
        """
        可以通过 wb.sheetnames 获取列表
        :return:
        """
        for sheet in self.wb:
            yield sheet

    def get_sheet_copy(self):
        """
        得到当前sheet的一个副本
        :return:
        """
        return self.wb.copy_worksheet(self.ws)

    def get_ws_value(self, key):
        """
        如 A4
        :param key:
        :return:
        """
        return self.ws[key].value

    def set_ws_value(self, key, value):
        self.ws[key] = value
        return self.ws[key]

    def get_sheet_value_cell(self, row, colum, value):
        """
        使用cell方法访问指定单元格如果没有会创建
        >>> for i in range(1,101):
        ...        for j in range(1,101):
        ...            ws.cell(row=i, column=j)
        将在内存中创建100x100个单元格，无需任何操作。
        :param row:
        :param colum:
        :param value:
        :return:
        """
        return self.ws.cell(row, colum, value)

    def get_sheet_value_slicing(self, index1, index2=None):
        """

        >>> colC = ws['C']
        >>> col_range = ws['C:D']
        >>> row10 = ws[10]
        >>> row_range = ws[5:10]

        :param index1: 数字或字母
        :param index2:
        :return: 通过传入的参数获取多个 tuple结果 比如 传入 A C 将返回A B C 3个tuple元组
        也可传入数字 传入数字将返回行，传入字母将返回列 也是以元组的方式返回
        注意 迭代元组得到 cell对象 可以 通过 cell.value得到值cell.row，cell.column得到行和列
        """
        if index2 is None:
            return self.ws[index1]
        return self.ws[index1:index2]

    def get_sheet_iter_rows(self, min_row, max_col, max_row):
        for row in self.ws.iter_rows(min_row, max_col, max_row):
            for cell in row:
                yield cell

    def get_sheet_iter_cols(self, min_row, max_col, max_row):
        for row in self.ws.iter_cols(min_row, max_col, max_row):
            for cell in row:
                yield cell

    def get_sheet_all_rows(self, is_row=True):
        """
        得到所有行和列 使用 tuple()查看对象
        :return:
        """
        if is_row:
            return self.ws.rows
        else:
            return self.ws.columns

    def set_sheet_cell_value(self, cell, value):
        """
        为单元格设置一个值
        :param cell:
        :param value:
        :return:
        """
        cell.value = value
        print(cell.value)

    def save_wb(self, excelname):
        """
        保存为excel文件
        :param excelname:
        :return:
        """
        self.wb.save(excelname)

    def save_stream(self):
        """
        保存到流
        :return:
        """
        with NamedTemporaryFile() as tmp:
            self.wb.save(tmp.name)
            tmp.seek(0)
            stream = tmp.read()
        return stream

    def save_wb_template(self, excelname, templatename):
        """
        >>> wb = load_workbook('document.xlsx')
        >>> wb.template = True
        >>> wb.save('document_template.xltx')
        您可以指定属性template = True，以将工作簿另存为模板：
        或者将此属性设置为False（默认），以保存为文档
        >>> wb = load_workbook('document_template.xltx')
        >>> wb.template = False
        >>> wb.save('document.xlsx', as_template=False)
        :param excelname:
        :return:
        """

        self.wb.template = True
        self.wb.save(excelname)

    def max_column(self):
        """
        所有列
        :return:
        """
        return self.ws.max_column

    def max_row(self):
        """
        所有行
        :return:
        """
        return self.ws.max_row

    def save_excel(self, filename):
        """
        保存excel文件
        :param filename:
        :return:
        """
        self.wb.save(filename)

    def get_font(self):
        """
        name 字符串 字体名称，例如'Calibri'或'Times New Roman'

        size 整数 字体大小

        bold 布尔 True，粗体字体

        italic 布尔 True，斜体字体
        :return:
        """
        fontObj1 = Font(bold=True)
        return fontObj1

    def get_Alignment(self):
        al = Alignment(horizontal="center")
        return al

    def set_font(self, fontobj, alignment, cells=[]):
        """
        :param cell:  A1
        :param fontobj:
        :return:
        """
        for cell in cells:
            self.ws[cell].font = fontobj
            self.ws[cell].alignment = alignment



import sys
import traceback

import pypyodbc as pypyodbc


class BaseODBC(object):

    def __init__(self, MdbFile):
        self.connStr = r'Driver={Microsoft Access Driver (*.mdb, *.accdb)};Dbq=%s;' % MdbFile
        # print('connStr:' + connStr)
        self.conn = pypyodbc.connect(self.connStr)

    def get_tables(self):
        """
        获取表名
        :return:
        """
        # print(self.cur.tables())
        for table in self.cur.tables():
            yield table

    def get_description(self,sql):
        """
        获取查询结果的字段信息
        sql = "select * from `journal`"
        :param sql:
        :return:
        """

        self.cur.execute(sql)
        for row in self.cur.description:
            yield row

    def select_all(self,sql):
        """
        查询语句执行
        :param sql:
        :return:
        """
        self.cur.execute(sql)
        for row in self.cur.fetchall():
            yield row

    def select_yield(self,sql):
        """
        查询语句执行
        :param sql:
        :return:
        """
        for row in self.cur.execute(sql):
            yield row


    def get_cur(self):
        """
        获取游标
        :return:
        """
        self.cur = self.conn.cursor()
        return self.cur

    def excsql(self, sql, errExit=True):
        """
        执行sql
        :param sql:
        :param errExit:
        :return:
        """
        print(sql)
        try:
            self.cur.execute(sql)
            self.conn.commit()
        except:
            print(traceback.format_exc())
            if errExit:
                sys.exit(-1)

    def exc_list_sql(self, listsql, errExit=True):
        """
        传入一个sql的列表，循环执行
        :param listsql:
        :param errExit:
        :return:
        """
        for sql in listsql:
            print('input sql:' + sql)
            try:
                self.cur.execute(sql)
            except:
                print(traceback.format_exc())
                if errExit:
                    sys.exit(-1)
        self.conn.commit()

    def close_all(self):
        """
        关闭链接
        :return:
        """
        self.cur.close()
        self.conn.close()



import urllib.parse as parse

from re_common.baselibrary.utils.basestring import BaseString


class BaseUrl(BaseString):

    @classmethod
    def urlQuery2Dict(cls, url):
        """
        url中的参数转换成dict 字典类型方便读取
        h还有另一种 方式
        dict(parse.parse_qsl(parse.urlparse(url).query))
        :param url:
        :return:
        """
        query = parse.urlparse(url).query
        return dict([(k, v[0]) for k, v in parse.parse_qs(query).items()])

    @classmethod
    def urlPath2List(cls, url):
        """
        获取路由并返回一个列表
        :param url:
        :return:
        """
        listPath = parse.urlparse(url).path.split("/")
        try:
            listPath.remove("")
        except:
            pass
        return listPath

    @classmethod
    def urlreplaceend(cls, url, endstrins):
        list_url = url.split("/")
        list_url.pop()
        list_url.append(endstrins)
        url = "/".join(list_url)
        return url

    @classmethod
    def urlNetloc(cls, url):
        """
        获取网址
        :param url:
        :return:
        """
        return parse.urlparse(url).netloc

    @classmethod
    def urlScheme(cls, url):
        """
        获取网站的请求方式 http or https
        :param url:
        :return:
        """
        return parse.urlparse(url).scheme

    @classmethod
    def urlQuery2List(cls, url):
        """
        urllib.parse.parse_qs 返回字典
        {'CID': ['quickSearchCitationFormat'], 'database': ['1'], 'SEARCHID': ['f84e4d9aMa3f4M4e2aMae22M912f5e06a259'], 'intialSearch': ['true'], 'angularReq': ['true'], 'usageOrigin': ['searchform'], 'usageZone': ['quicksearch'], 'implicit': ['false'], 'isFullJsonResult': ['true'], 'startTime': ['1529473818439'], 'endTime': ['1529473818477']}
        urllib.parse.parse_qsl 返回列表
        [('CID', 'quickSearchCitationFormat'), ('database', '1'), ('SEARCHID', 'f84e4d9aMa3f4M4e2aMae22M912f5e06a259'), ('intialSearch', 'true'), ('angularReq', 'true'), ('usageOrigin', 'searchform'), ('usageZone', 'quicksearch'), ('implicit', 'false'), ('isFullJsonResult', 'true'), ('startTime', '1529473818439'), ('endTime', '1529473818477')]
        :param url:
        :return:
        """
        return parse.parse_qsl(parse.urlparse(url).query)

    @classmethod
    def urlQuery2dic(cls, url):
        """
        urllib.parse.parse_qs 返回字典
        {'CID': ['quickSearchCitationFormat'], 'database': ['1'], 'SEARCHID': ['f84e4d9aMa3f4M4e2aMae22M912f5e06a259'], 'intialSearch': ['true'], 'angularReq': ['true'], 'usageOrigin': ['searchform'], 'usageZone': ['quicksearch'], 'implicit': ['false'], 'isFullJsonResult': ['true'], 'startTime': ['1529473818439'], 'endTime': ['1529473818477']}
        urllib.parse.parse_qsl 返回列表
        [('CID', 'quickSearchCitationFormat'), ('database', '1'), ('SEARCHID', 'f84e4d9aMa3f4M4e2aMae22M912f5e06a259'), ('intialSearch', 'true'), ('angularReq', 'true'), ('usageOrigin', 'searchform'), ('usageZone', 'quicksearch'), ('implicit', 'false'), ('isFullJsonResult', 'true'), ('startTime', '1529473818439'), ('endTime', '1529473818477')]
        :param url:
        :return:
        """
        return parse.parse_qs(parse.urlparse(url).query)

    @classmethod
    def urlencode(cls, dic):
        """
        将dic型的url参数进行编码 如
        {'CID': 'quickSearchCitationFormat', 'database': '1', 'SEARCHID': 'f84e4d9aMa3f4M4e2aMae22M912f5e06a259', 'intialSearch': 'true', 'angularReq': 'true', 'usageOrigin': 'searchform', 'usageZone': 'quicksearch', 'implicit': 'false', 'isFullJsonResult': 'true', 'startTime': '1529473818439', 'endTime': '1529473818477'}
        :param dic:
        :return:
        """
        return parse.urlencode(dic)

    @classmethod
    def urldecode(self, strings):
        """
        url解码
        :param strings:
        :return:
        """
        return parse.unquote(strings)

    @classmethod
    def dicts_to_url(cls, dic, url=None):
        """
        将字典转换成get的url参数部分
        :param dic:
        :return:
        """
        parastring = ""
        for key, value in dic.items():
            parastring += str(key) + "=" + str(value) + "&"
        parastring = parastring[:-1]
        if url:
            return url + "?" + parastring
        return parastring

# url = "https://www.engineeringvillage.com/search/results/quick.url?CID=quickSearchCitationFormat&database=1&SEARCHID=f84e4d9aMa3f4M4e2aMae22M912f5e06a259&intialSearch=true&angularReq=true&usageOrigin=searchform&usageZone=quicksearch&implicit=false&isFullJsonResult=true&startTime=1529473818439&endTime=1529473818477"
# dic = BaseUrl.urlQuery2Dict(url)
# print(dic)
# BaseUrl.urlencode(url)



from ftplib import FTP


class BaseFtp(object):
    def __init__(self,encoding='utf-8'):
        self.ftp = FTP()
        self.ftp.encoding = encoding

    def conn_ftp(self,FtpHost):
        self.ftp.connect(FtpHost, 21)
        return self

    def login(self,username,passwd):
        self.ftp.login(username,passwd)

    def cwd(self,dires):
        """
        执行ftp命令
        :param dires:
        :return:
        """
        self.ftp.cwd(dires)  # change remote work dir

    def getfilesize(self,remotefile):
        return self.ftp.size(remotefile)

    def voidcmd(self):
        """
        :return:
        """
        self.ftp.voidcmd('TYPE I')


###########################################
# 同项目调用基础包
import datetime
import gzip
import json
import os
import sys
import time

filepath = os.path.abspath(__file__)
pathlist = filepath.split(os.sep)
pathlist = pathlist[:-3]
TopPath = os.sep.join(pathlist)
sys.path.insert(0, TopPath)
print(TopPath)
############################################

from re_common.baselibrary.utils.basemssql import BaseMsSql
from re_common.baselibrary.utils.basefile import BaseFile
from re_common.baselibrary.utils.basetime import BaseTime
bt = BaseTime()

host = "127.0.0.1"
user = "sa"
pwd = "xujiang1994323"
db = "patData"
charset = "utf8"

basemssql = BaseMsSql(host, user, pwd, db, charset, as_dict=True)
basemssql.conn()
basemssql.exec_select_query("select * from [dbo].[New_pattzb]")
outPathFile = r"F:\db3\patnetjson_big\jss_patent.big_json.gz"
i = 0
size = 10000
count = 2000000
outPathFile = BaseFile.get_new_filename(outPathFile, sign=".")
start = time.time()
end = False
while True:
    with gzip.open(outPathFile, 'wb') as f:
        while True:
            a = basemssql.cur.fetchmany(size=size)
            if not a:
                print(i)
                print(int(time.time() - start))
                print("break")
                end = True
                break
            for row in a:
                dicts = {}
                for k,v in row.items():
                    # print(k)
                    try:
                        v = v.encode('latin-1').decode('gbk')
                    except:
                        pass
                    if isinstance(v, datetime.datetime):
                        v = bt.datetime_to_string(v, "%Y-%m-%d %H:%M:%S")
                    dicts[k] = v
                    # print(type(v))
                # print(dicts)
                line = json.dumps(dicts,ensure_ascii=False)+"\n"
                # print(line)
                lines = line.encode(encoding="utf8")
                f.write(lines)
            i = i + size
            print(i)
            print(int(time.time() - start))
            if i >= count:
                outPathFile = BaseFile.get_new_filename(outPathFile, sign=".")
                i=0
                break
    if end:
        break






from re_common.baselibrary.utils.baselist import BaseList


class StringToDicts(object):

    def __init__(self):
        pass

    def string_to_dicts_by_equal(self, strings):
        """
        将string 按行分组 然后每组内使用等号分割 前面为字典的键 后面为字典的值
        strings = ```
        host = 192.168.31.209
        user = root
        passwd = vipdatacenter
        db = data_gather_record
        port = 3306
        chartset = utf8
        ```
        :param strings:
        :return:
        """
        baselistobj = BaseList()
        lists = strings.split("\n")
        lists = baselistobj.remove_null(lists)
        lists = baselistobj.remove_blank_space(lists)
        lists = baselistobj.clean_space_first_end(lists)
        dicts = {}
        for item in lists:
            key = item.split("=")[0].strip()
            values = item.split("=")[1].strip()
            dicts[key] = values
        return dicts



from .baseabs import BaseAbs
from .database.mbuilder import MysqlBuilderAbstract
from .readconfig.ini_config import IniConfig
from .utils.mylogger import MLogger

__all__ = ["BaseAbs", "MysqlBuilderAbstract", "IniConfig", "MLogger"]



from typing import List

from motor.motor_asyncio import AsyncIOMotorClient


class BaseMotor(object):

    def __init__(self):
        pass

    def AsyncIOMotorClient(self, uri, dbname):
        """
        异步链接mongo客户端
        :param uri:
        :param dbname:
        :return:
        """
        self.connection = AsyncIOMotorClient(uri)
        self.db = self.connection[dbname]

    def get_col(self, colname):
        """
        获取表
        :param colname:
        :return:
        """
        self.col = self.db[colname]

    async def select_yield(self, query={}):
        """
        异步查询
        :param query:
        :return:
        """
        async for doc in self.col.find(query):
            yield doc

    async def select(self, query={}):
        """
        异步查询
        :param query:
        :return:
        """
        docs = self.col.find(query)
        lists = await docs.to_list(None)
        return lists

    async def select_one(self, query={}):
        """
        异步查询一条
        :param query:
        :return:
        """
        doc = await self.col.find_one(query)
        # print(doc)
        return doc

    async def insert_many(self, lists: List[dict], ordered=False, *args, **kwargs):

        """
        插入一个列表（列表内为dict类型）
        :param lists:
        :param ordered: 如果某一条出现错误 设置为False会继续处理其他数据，默认为true
        :param args:
        :param kwargs:
        :return:
        """
        result = await self.col.insert_many(lists, ordered=ordered, *args, **kwargs)
        print('inserted %d docs' % (len(result.inserted_ids),))
        return result

    async def insert_one(self, dicts):
        """
        异步插入一条
        :param dicts:
        :return:
        """
        result = await self.col.insert_one(dicts)
        # print('result %s' % repr(result.inserted_id))

    async def replace_one(self, dicts, iddicts=None):
        if iddicts is None:
            iddicts = {"_id": dicts["_id"]}
        result = await self.col.replace_one(iddicts, dicts)
        print('result %s' % repr(result.modified_count))

    async def find(self, doc_hook, query=None, feild=None):
        """
        异步查询
        for document in await cursor.to_list(length=100):
        """
        async for doc in self.col.find(query, feild):  # 查询所有文档
            await doc_hook(doc)

    async def update(self, query, sets):
        result = await self.col.update_one(query, {'$set': sets})
        # print('updated %s document' % result.modified_count)

    async def run_common(self, doc_hook, commons):
        result = await self.db.command(commons)
        doc_hook(result)

    async def delete_many(self, query):
        """

        :param query: 一个字典 按照条件删除
        :return:
        """
        result = await self.col.delete_many(query)

    async def delet_one(self, query):
        """
        删除一条数据
        :param query_id:
        :return:
        """
        result = await self.col.delete_one(query)

#
# bs = BaseMotor()
# bs.AsyncIOMotorClient(
#     "mongodb://cjrw:vipdatacenter@192.168.31.243:32920,192.168.31.208:32920,192.168.31.206:32920/testdb.jss_pantent_detail?authSource=testdb",
#     "testdb")
# bs.get_col("jss_pantent_detail")
#
# print(bs.select_one({"_id":"Patent_AU19920025315"}))
#
# i = 0
# lists = []
# start_time = time.time()
# for file in BaseDir.get_dir_all_files(r"F:\fun2\gz"):
#     print(file)
#     for line in BaseGzip(100).read_gz_file(file):
#         i = i + 1
#         line = line.strip()
#         dicts = json.loads(line)
#         dicts["export_stat"] = 0
#         dicts["_id"] = dicts["rawid"]
#         del dicts["rawid"]
#         lists.append(dicts)
#         if i % 100000 == 1:
#             print(i)
#             try:
#                 asyncio.get_event_loop().run_until_complete(bs.insert_many(lists))
#             except BulkWriteError as e:
#                 print(e.args)
#                 # print(e.details)
#             lists.clear()
#             print(time.time()-start_time)
#
# try:
#     asyncio.get_event_loop().run_until_complete(bs.insert_many(lists))
# except BulkWriteError as e:
#     print(e.args)
#     # print(e.details)
# lists.clear()
# print(time.time() - start_time)
# print(i)
# asyncio.get_event_loop().run_until_complete(bs.select_one({"_id": "Patent_AU19920025315"}))



import asyncio
import functools
import sys
import time
import traceback
import warnings
from functools import wraps

# https://www.jianshu.com/p/ee82b941772a
import aiohttp.client_exceptions
import requests
from aiohttp import ClientProxyConnectionError

from re_common.baselibrary.utils.core.requests_core import MsgCode


def InterVal(start_time, interval):
    '''
    执行函数必须间隔多少时间,没有到指定的时间函数就不会被执行
    :param start_time: 传入开始时间
    :param interval: 秒数
    :return: 返回执行结果  否则返回 None
    '''

    def dewrapper(func):
        """
        它能把原函数的元信息拷贝到装饰器里面的 func 函数中。
        函数的元信息包括docstring、name、参数列表等等。可以尝试去除@functools.wraps(func)，
        你会发现test.__name__的输出变成了wrapper。
        """

        @wraps(func)
        def wrapper(*args, **kwargs):
            start = start_time
            time_now = int(time.time())
            if time_now - start > interval:
                result = func(*args, **kwargs)
                return result
            else:
                return None

        return wrapper

    return dewrapper


def timethis(func):
    """
    函数执行的时间差
    :param func:
    :return:
    """

    @wraps(func)
    def wrapper(*args, **kwargs):
        start = time.time()
        result = func(*args, **kwargs)
        end = time.time()
        print(func.__name__, end - start)
        return result

    return wrapper


def try_except(callback=None, is_print=True):
    """
    使用装饰器使用try_except,并用callback回收错误信息
    :param func:
    :return:
    """

    def dewrapper(func):

        @wraps(func)
        def wrapper(*args, **kwargs):
            try:
                return func(*args, **kwargs)
            except BaseException as e:
                if callback is not None:
                    callback(*sys.exc_info(), *args, **kwargs)
                if is_print:
                    print(traceback.format_exc())
                return traceback.format_exc()

        return wrapper

    return dewrapper


def try_except2(callback=None, is_print=True):
    """
    使用装饰器使用try_except,并用callback回收错误信息
    :param func:
    :return:
    """

    def dewrapper(func):

        @wraps(func)
        def wrapper(*args, **kwargs):
            try:
                return func(*args, **kwargs)
            except BaseException as e:
                if callback is not None:
                    callback(*sys.exc_info(), *args, **kwargs)
                if is_print:
                    print(traceback.format_exc())
                return False, traceback.format_exc()

        return wrapper

    return dewrapper


def try_except2_async(callback=None, is_print=True):
    """
    使用装饰器使用try_except,并用callback回收错误信息
    异步函数专用装饰器
    :param func:
    :return:
    """

    def dewrapper(func):

        @functools.wraps(func)
        async def wrapper(*args, **kwargs):
            try:
                return await func(*args, **kwargs)
            except BaseException as e:
                if callback is not None:
                    callback(*sys.exc_info(), *args, **kwargs)
                if is_print:
                    print(traceback.format_exc())
                return False, traceback.format_exc()

        return wrapper

    return dewrapper


# https://blog.csdn.net/qq_39314099/article/details/83822593
def deprecated(message):
    def deprecated_decorator(func):
        @wraps(func)
        def deprecated_func(*args, **kwargs):
            warnings.warn("{} is a deprecated function. {}".format(func.__name__, message),
                          category=DeprecationWarning,
                          stacklevel=2)
            warnings.simplefilter('default', DeprecationWarning)
            return func(*args, **kwargs)

        return deprecated_func

    return deprecated_decorator


def request_try_except(func):
    """
    拦截request请求错误
    :param func:
    :return:
    """

    @wraps(func)
    def wrapTheFunction(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except requests.exceptions.ReadTimeout as e:
            dicts = {"code": MsgCode.TIME_OUT_ERROR,
                     "msg": "time out, {}".format(repr(e))}
            return False, dicts
        except requests.exceptions.ProxyError as e:
            dicts = {"code": MsgCode.PROXY_ERROR,
                     "msg": "proxy error, {}".format(repr(e))}
            return False, dicts
        except:
            dicts = {"code": MsgCode.ON_KNOW,
                     "msg": traceback.format_exc()}
            return False, dicts

    return wrapTheFunction


def aiohttp_try_except(func):
    """
    拦截aiohttp 异步请求错误
    :param func:
    :return:
    """

    @wraps(func)
    async def wrapTheFunction(*args, **kwargs):
        try:
            return await func(*args, **kwargs)
        except ClientProxyConnectionError as e:
            dicts = {"code": MsgCode.PROXY_ERROR,
                     "msg": "proxy error, {}".format(repr(e))}
            return False, dicts
        except asyncio.exceptions.TimeoutError as e:
            dicts = {"code": MsgCode.TIME_OUT_ERROR,
                     "msg": "time out, {}".format(repr(e))}
            return False, dicts
        except aiohttp.client_exceptions.ClientPayloadError as e:
            dicts = {"code": MsgCode.PAYLOAD_ERROR,
                     "msg": "非法的压缩格式，错误的chunk编码，数据不足Content-length, {}".format(repr(e))}
            return False, dicts
        except Exception as e:
            dicts = {"code": MsgCode.ON_KNOW,
                     "msg": traceback.format_exc()}
            return False, dicts

    return wrapTheFunction


def mongodb_try_except(func):
    """
    拦截mongodb执行错误
    :param func:
    :return:
    """

    @wraps(func)
    async def wrapTheFunction(*args, **kwargs):
        try:
            return await func(*args, **kwargs)
        except Exception as e:
            dict_ = {
                "status": "FAILED",
                "msg_code": MsgCode.NO_RESOURCE,
                "msg": "保存到数据库失败",
                "data": {"err": repr(e)}
            }
            return False, dict_

    return wrapTheFunction


def retry_func(retry_times=3, sleep_time=1):
    """函数重复执行次数"""

    def retry_decorator(func):
        @functools.wraps(func)
        def wrapper_func(*args, **kwargs):
            flag = 0
            while flag < retry_times:
                res = func(*args, **kwargs)
                res_bool = res
                if tuple(res) == tuple:
                    res_bool = res[0]
                if not res_bool:
                    flag += 1
                    # logger.warning("{0} execute {1} times".format((func.__name__), flag))
                    time.sleep(sleep_time)
                    continue
                else:
                    return res

        return wrapper_func

    return retry_decorator


def retry_func_async(retry_times=3, sleep_time=1):
    """函数重复执行次数"""

    def retry_decorator(func):
        @functools.wraps(func)
        async def wrapper_func(*args, **kwargs):
            flag = 0
            while flag < retry_times:
                res = await func(*args, **kwargs)
                res_bool = res
                if tuple(res) == tuple:
                    res_bool = res[0]
                if not res_bool:
                    flag += 1
                    # logger.warning("{0} execute {1} times".format((func.__name__), flag))
                    time.sleep(sleep_time)
                    continue
                else:
                    return res

        return wrapper_func

    return retry_decorator



import time

import pymongo

from re_common.baselibrary.utils.basepymongo import BasePyMongo

basemongo = BasePyMongo("mongodb://cjrw:vipdatacenter@192.168.31.166:27017/")
basemongo.use_db("collection")
basemongo.create_col("jss_w_f_panternt_page")
starttime = time.time()
print(basemongo.delete_col())

print(time.time() - starttime)
# dicts = {
#   "cid": "string",
#   "pagenum": "string",
#   "down_date": "string",
#   "html": "string",
#   "types": "string"
# }
# basemongo.insert_one(dicts)



###########################################
# 同项目调用基础包
import os
import sys

filepath = os.path.abspath(__file__)
pathlist = filepath.split(os.sep)
pathlist = pathlist[:-3]
TopPath = os.sep.join(pathlist)
sys.path.insert(0, TopPath)
print(TopPath)
############################################
from re_common.baselibrary.tools.merge_file import MergeFile

if __name__ == '__main__':
    old_path = r'E:\work\pnasjournal\big_json\20200116'
    new_path = r'E:\work\pnasjournal\big_json\new'
    a = MergeFile()
    # a.line_all_2_one(old_path,new_path)
    a.block_all_2_one(old_path, new_path, dst_size=1024 * 1024 * 200, block_size=200 * 1024 * 1024)



 from re_common.baselibrary.tools.stringtodicts import StringToDicts
from re_common.facade.mysqlfacade import MysqlUtiles

"""
本方法主要用于provider 变化,有时候我们可能需要更新209的provider
用该方法更新方便快捷
"""
strings = """
        host = 192.168.31.209
        user = root
        passwd = vipdatacenter
        db = data_gather_record
        port = 3306
        chartset = utf8
        """

dicts_change = {"key为原来的": "values为现在的"}

dicts = StringToDicts().string_to_dicts_by_equal(strings)
mysqlutils = MysqlUtiles("", "", builder="MysqlBuilderForDicts", dicts=dicts)
mysqlutils.ExeSqlToDB("SET FOREIGN_KEY_CHECKS=0;")
for key, values in dicts_change:
    sql1 = "update `data_gather_record`.`task` set `provider` = '{}' WHERE `provider` = '{}';".format(values, key)
    sql2 = "update `data_gather_record`.`updating` set `provider` = '{}' WHERE `provider` = '{}';".format(values, key)
    mysqlutils.ExeSqlToDB(sql1)
    mysqlutils.ExeSqlToDB(sql2)

mysqlutils.ExeSqlToDB("SET FOREIGN_KEY_CHECKS=1;")



import copy


class BaseDicts(object):

    @classmethod
    def removeDictsNone(self, dicts: dict) -> dict:
        """
        去除字典中值为None的键值
        :param dicts:
        :return:
        """
        return {key: value for key, value in dicts.items() if value is not None}

    @classmethod
    def removeDictsStringNull(self, dicts: dict) -> dict:
        """
        去除字典中值为''的键值
        :param dicts:
        :return:
        """
        return {key: value for key, value in dicts.items() if value != ""}

    @classmethod
    def removeDictsAllNone(self, dicts: dict) -> dict:
        """
        去除字典中值为'' 和 None 的键值
        :param dicts:
        :return:
        """
        return {key: value for key, value in dicts.items() if value != "" and value is not None}

    @classmethod
    def sortkeys(self, dicts, reverse=False):
        """
        默认升序排序，加  reverse = True 指定为降序排序
        通过keys 对dicts 排序
        经过测试是新的列表
        :return:
        """
        return {k: dicts[k] for k in sorted(dicts.keys(), reverse=reverse)}

    @classmethod
    def sortvalues(self, dicts, reverse=False):
        """
        默认升序排序，加  reverse = True 指定为降序排序
        d[1] 为值　ｄ[0]　为键
        d 为元组　为dicts的键值
        通过　values 对dicts 排序
        :param dicts:
        :return:
        """
        return {k: v for k, v in sorted(dicts.items(), key=lambda d: d[1], reverse=reverse)}

    @classmethod
    def is_key_have(cls, dicts, key):
        """
        判断key 是否存在,但只能判断一个层次
        :param dicts:
        :param key:
        :return:
        """
        if key in dicts.keys():
            return True
        else:
            return False

    @classmethod
    def is_more_key_have(cls, dicts, keys=[]):
        """
        判断多个key 是否存在　可以有更深的层次
        :param dicts:
        :param keys: ["a.b","c.d"]
        :return:
        """
        for item in keys:
            if item.find("."):
                allstrings = ""
                for key in item.split("."):
                    allstrings = allstrings + '["{}"]'.format(key)
                try:
                    eval("dicts" + allstrings)
                except:
                    return False
            else:
                if item not in dicts.keys():
                    return False
        return True

    @classmethod
    def get_recursive_dict(cls, dict_a, key, call_back):
        if isinstance(dict_a, dict):
            for key, value in dict_a.items():
                if isinstance(value, list) or isinstance(value, tuple) or isinstance(value, dict):
                    dict_a[key] = cls.get_recursive_dict(value, key, call_back)
                else:
                    dict_a[key] = call_back(key, value)

        elif isinstance(dict_a, list):
            # 如果列表中存在一个不是字典就不需要遍历了
            if dict_a == []:
                return call_back(key, dict_a)
            is_true = True
            for i in dict_a:
                if isinstance(i, dict):
                    is_true = True
                else:
                    is_true = False
                    continue
            if is_true:
                dict_temp = []
                for value in dict_a:
                    dict_temp.append(cls.get_recursive_dict(value, key, call_back))
                return dict_temp
            else:
                return call_back(key, dict_a)
        elif isinstance(dict_a, tuple):
            # 如果存在元组转成列表使用
            return cls.get_recursive_dict(list(dict_a), key, call_back)
        else:
            assert False, "传入类型错误：{}".format(type(dict_a))
        return dict_a



from re_common.baselibrary.utils.baseavro import BaseAvro

id_set = set()
for line in BaseAvro().read_line_yeild(r"F:\fun2\avro"):
    id_set.add(line["key"])

print(len(id_set))
lines = ""

with open(r"F:\fun2\avro1.txt", 'w', encoding="utf-8") as f:
    for id in id_set:
        lines = id + "\n"
        f.write(lines)



from re_common.baselibrary.utils.myredisclient import MyRedis

myredis = MyRedis("./config.ini")
myredis.set_redis_from_config(sesc="redis_test")
myredis.conn_redis()
myredis.get_pipeline()
# myredis.mset({"k1": "v1", "k2": "v2"})
a = myredis.hset("hash1", "k1", "v1")

print(a)



import time

from re_common.baselibrary.mthread.MThreadingRun import MThreadingRun2
from re_common.baselibrary.mthread.mythreading import ThreadInfo
from re_common.baselibrary.utils.basedir import BaseDir
from re_common.baselibrary.utils.basefile import BaseFile
from re_common.baselibrary.utils.basequeue import BaseQueue
from re_common.facade.now import get_streamlogger, get_filelogger


def copy_one_file(fullname, sources_path, dst_path):
    """
    复制文件 不会删除
    :param fullname:
    :param sources_path:
    :param dst_path:
    :return:
    """
    fullname_size = BaseFile.get_file_size(fullname)
    # 替换前面的路径得到新文件的路劲
    dst_file = fullname.replace(sources_path, dst_path)
    # 目录不存在就创建目录
    dst_dir = BaseDir.get_file_dir_absolute(dst_file)
    BaseDir.create_dir(dst_dir)

    if BaseFile.is_file_exists(dst_file):
        dst_dir_size = BaseFile.get_file_size(dst_file)
        if fullname_size == dst_dir_size:
            return True
    # 复制文件
    BaseFile.copy_file_to_file(fullname, dst_file)
    dst_dir_size = BaseFile.get_file_size(dst_file)
    # 文件大小一致才删除源文件
    # 大小一致性对比不能用于windows 和 liunx之间的传输，两者会存在差异性
    print(fullname_size,dst_dir_size)
    if fullname_size == dst_dir_size:
        return True
    else:
        return False


def move_one_file(fullname, sources_path, dst_path):
    """
    移动文件，会删除源文件
    :param fullname:
    :param sources_path:
    :param dst_path:
    :return:
    """
    fullname_size = BaseFile.get_file_size(fullname)
    # 替换前面的路径得到新文件的路劲
    dst_file = fullname.replace(sources_path, dst_path)
    # 目录不存在就创建目录
    dst_dir = BaseDir.get_file_dir_absolute(dst_file)
    BaseDir.create_dir(dst_dir)
    if BaseFile.is_file_exists(dst_file):
        dst_dir_size = BaseFile.get_file_size(dst_file)
        if fullname_size == dst_dir_size:
            BaseFile.remove_file(fullname)
    # 复制文件
    BaseFile.copy_file_to_file(fullname, dst_file)
    dst_dir_size = BaseFile.get_file_size(dst_file)
    # 文件大小一致才删除源文件
    if fullname_size == dst_dir_size:
        BaseFile.remove_file(fullname)


def move_file(sources_path, dst_path):
    """
    移动文件，会删除源文件
    :param sources_path:
    :param dst_path:
    :return:
    """
    k = 0
    for fullname in BaseDir.get_dir_all_files(sources_path):
        k += 1
        move_one_file(fullname, sources_path, dst_path)
        if k % 1000 == 0:
            print("已经拷贝{}个文件".format(k))
    print("已经拷贝{}个文件".format(k))


class MoveFile(object):

    def __init__(self):
        self.sources_path = ''
        self.dst_path = ''


class MoveFileThread(MThreadingRun2):
    def __init__(self, num, mf):
        super(MoveFileThread, self).__init__(num)
        self.mf = mf
        self.k = 0
        self.file_logger = get_filelogger(r"F:\fun2\log")
        self.thread_pool.work_queue = BaseQueue(1000)

    def create_dir(self):
        """
        先创建所有目录,防止在多线程中创建目录冲突
        :return:
        """
        for dir in BaseDir.get_dir_all_dir(self.mf.sources_path):
            new_dir = dir.replace(self.mf.sources_path, self.mf.dst_path)
            print(new_dir)
            BaseDir.create_dir(new_dir)

    def set_task(self, threadval, *args, **kwargs):
        self.create_dir()
        for fullname in BaseDir.get_dir_all_files(self.mf.sources_path):
            self.k = self.k + 1
            self.add_job(self.func, fullname)
            if self.k % 1000 == 0:
                print("已经拷贝{}个文件".format(self.k))
        while self.get_thread_stat():
            time.sleep(1)

    def get_thread_stat(self):
        """
        如果是False代表所有的线程都在等待任务，说明所有工作都已经完成
        :return:
        """
        for k, v in self.thread_pool.thread_pool_dicts.items():
            threadinfo = v
            thread = threadinfo.get_thread()
            if thread.runstatus is not False:
                return True
        return False

    def deal_results(self, threadval, *args, **kwargs):
        time.sleep(60)

    def setProxy(self, threadval, proxysList=None):
        time.sleep(60)

    def is_break(self):
        time.sleep(5)
        if self.thread_pool.work_queue.is_empty() and self.thread_pool.result_queue.is_empty() and not self.get_thread_stat():
            for k, v in self.thread_pool.thread_pool_dicts.items():
                if k == self.etn.taskthreadname:
                    threadinfo = v
                    thread = threadinfo.get_thread()
                    if thread.is_alive():
                        return False
            return True
        else:
            return False

    def thread_pool_hook(self, threadinfo: ThreadInfo):
        # 设置代理线程不重启，默认会重启
        if threadinfo.get_thread_name() == self.etn.taskthreadname:
            threadinfo.set_is_restart(False)
        return {}

    def fun(self, threadval, *args, **kwargs):
        fullname = args[0]
        copy_one_file(fullname, self.mf.sources_path, self.mf.dst_path)
            # self.file_logger.info(fullname + "\n")


if __name__ == "__main__":
    mf = MoveFile()
    mf.sources_path = r"F:\fun2\test_gif"
    # mf.dst_path = r"F:\fun2\test_gif2"
    mf.dst_path = r"\\192.168.31.123\home\cjvip\qinym\soopat\image"
    # mf.dst_path = r"\\192.168.31.177\down_data\test"

    mft = MoveFileThread(30, mf)
    mft.run()



# -*- coding: utf-8 -*-
# @Time    : 2020/9/16 13:18
# @Author  : suhong
# @File    : mcss.py
# @Software: PyCharm
from parsel import Selector


class MParsel(object):
    def __init__(self, html="", logger=None):
        if logger is None:
            from re_common.baselibrary import MLogger
            logger = MLogger().streamlogger
        self.logger = logger
        self.html = html
        if self.html:
            self.set_sel(self.html)

    def set_sel(self, html):
        self.html = html
        self.sel = Selector(html)

    def css_parsel(self, sel, css_selector={}):
        assert self.html, '解析html时html不能为空'
        dicts = {}
        parent = None
        if css_selector:
            # 保证 parent 存在时优先解析
            if "parent" in css_selector.keys():
                parent = sel.css(css_selector["parent"])
            for key, value in css_selector.items():
                if isinstance(value, dict):
                    if key == "children":
                        # assert parent, "parent 不存在"
                        list_c = []
                        for p_item in parent:
                            list_c.append(self.css_parsel(p_item, value))
                        dicts[key] = list_c
                    else:
                        dicts[key] = self.css_parsel(sel, value)
                    continue
                if key == "parent":
                    continue
                dicts[key] = sel.css(value).getall()
        return dicts

    def css_parsel_html(self, css_selector={}):
        if self.html != "" and css_selector:
            dict_ = dict()
            for key, value in css_selector.items():
                dict_[key] = self.sel.css(value).getall()
            return True, dict_
        else:
            return False, ""

    def xpath_parsel(self, sel, xpath_selector={}):
        assert self.html, '解析html时html不能为空'
        dicts = {}
        parent = None
        if xpath_selector:
            # 保证 parent 存在时优先解析
            if "parent" in xpath_selector.keys():
                parent = sel.xpath(xpath_selector["parent"])
            for key, value in xpath_selector.items():
                if isinstance(value, dict):
                    if key == "children":
                        # assert parent, "parent 不存在"
                        list_c = []
                        for p_item in parent:
                            list_c.append(self.xpath_parsel(p_item, value))
                        dicts[key] = list_c
                    else:
                        dicts[key] = self.xpath_parsel(sel, value)
                    continue
                if key == "parent":
                    continue
                dicts[key] = sel.xpath(value).getall()
        return dicts

    def xpath_parsel_html(self, html="", xpath_selector={}):
        if html != "" and xpath_selector:
            sel = Selector(html)
            dict_ = dict()
            for key, value in xpath_selector.items():
                dict_[key] = sel.xpath(value).getall()
            return True, dict_
        else:
            return False, ""

    def asd(self):
        pass



from re_common.baselibrary import BaseAbs
from re_common.baselibrary.utils.basehdfs import BaseHDFS


class DownHdfs(object):

    def __init__(self):
        self.toml_file = "config_down.toml"
        self.dicts = {}
        self.basehdfs = None

    def read_toml(self):
        dicts = BaseAbs.get_config_factory().toml_factory().set_config_path(
            self.toml_file).read_file_remove_bom().get_dicts()
        self.dicts = dicts
        return self

    def set_basehdfs(self, i=0):
        self.basehdfs = BaseHDFS()
        self.basehdfs.hdfsdir = self.dicts["PathPair"][i][0]
        self.basehdfs.localdir = self.dicts["PathPair"][i][1]
        self.basehdfs.user_name = self.dicts["UserName"]
        self.basehdfs.namenode = self.dicts["NameNode"]
        return self

    def down_files(self):
        self.basehdfs.get_client()
        self.basehdfs.get_all_files_num()
        self.basehdfs.get_all_files()

    def use(self):
        self.read_toml()
        for i in range(len(self.dicts["PathPair"])):
            self.set_basehdfs(i)
            self.down_files()


DownHdfs().use()



from re_common.baselibrary import BaseAbs
from re_common.baselibrary.utils.basehdfs import BaseHDFS


class HdfsTools(object):
    def __init__(self):
        self.toml_file = "config_down.toml"
        self.dicts = {}
        self.basehdfs = None

    def read_toml(self):
        dicts = BaseAbs.get_config_factory().toml_factory().set_config_path(
            self.toml_file).read_file_remove_bom().get_dicts()
        self.dicts = dicts
        return self

    def set_basehdfs(self, i=0):
        self.basehdfs = BaseHDFS()
        self.basehdfs.hdfsdir = self.dicts["PathPair"][i][0]
        self.basehdfs.localdir = self.dicts["PathPair"][i][1]
        self.basehdfs.user_name = self.dicts["UserName"]
        self.basehdfs.namenode = self.dicts["NameNode"]
        return self

    def builder(self):
        self.basehdfs.get_client()

    def create_dirs(self):
        self.basehdfs.get_client()
        print(self.basehdfs.mk_hdfs_dirs(r"/RawData/wanfang/qk/ref/big_json/20200421"))


    def exists(self):
        print(self.basehdfs.exists(r"/RawData/wanfang/qk/ref/big_json/20200421"))

    def run(self):
        self.read_toml()
        self.set_basehdfs()
        self.builder()
        self.exists()

HdfsTools().run()



import pika


# https://blog.csdn.net/songfreeman/article/details/50943603
class BasePika(object):

    def __init__(self, username=None, password=None, mqhost=None, virtual_host=None):
        self.username = username
        self.password = password
        self.conn = None
        self.host = mqhost
        self.virtual_host = virtual_host
        self.auto_ack = True

    def set_default(self):
        self.host = "192.168.31.79"
        self.virtual_host = "vhost_NetDataGather"
        self.username = "vip"
        self.password = "piv$*123"

    def connect_str(self,amqp_str):
        parameters = pika.URLParameters(amqp_str)
        self.conn = pika.BlockingConnection(parameters)

    def connect(self):
        """
        设置用户名 密码 进行连接
        :return:
        """
        credentials = pika.PlainCredentials(self.username, self.password)
        # parameters = pika.URLParameters('amqp://guest:guest@rabbit-server1:5672/%2F')
        # 可以通过将 heartbeat 设为 0，关闭 rabbitmq 的心跳检测
        parameters = pika.ConnectionParameters(host=self.host,
                                               virtual_host=self.virtual_host,
                                               credentials=credentials,
                                               heartbeat=0)
        self.conn = pika.BlockingConnection(parameters)

    def close(self):
        # 关闭消息队列
        self.conn.close()

    def create_channel(self):
        self.channel = self.conn.channel()

    def __del__(self):
        self.channel.close()
        self.conn.close()

    def random_queue_declare(self):
        """
        这样, result.method.queue 包含一个随机的队列名, 比如：看起来像 amq.gen-JzTY20BRgKO-HjmUJj0wLg.
        其次：
        一旦我们断开consumer连接，这个队列名将自动删除。这里有一个标识设置：
        :return:
        """
        return self.channel.queue_declare("", exclusive=True)

    def queue_declare(self, queue="hello", durable=False):
        """
        创建目的地队列hello 取消息时也可以调用
        取消息和发送消息都调用 保证队列存在，也保证了不管服务端还是客户端先启动都有队列
        durable True 为持久化
        :return:
        """
        return self.channel.queue_declare(queue=queue, durable=durable)

    def get_queue_size(self, queue="hello"):
        """
        获取某个队列的长度
        :param queue:
        :return:
        """
        queue = self.queue_declare(queue=queue, durable=True)
        return queue.method.message_count

    def get_properties(self):
        """
        与 queue_declare里的 durable = True 配合使用，
        设置给 easy_send_msg的properties
        :return:
        """
        return pika.BasicProperties(
            delivery_mode=2,  # 设置消息为持久化的
        )

    def easy_send_msg(self, exchange="", routing_key="hello", body="hello world", properties=None):
        """
        空字符串标识默认的或者匿名的exchange,如果存在routing_key, 消息路由到routing_key指定的队列中。
        routing_key 标识发送到哪个队列,就是服务器上的队列名
        body 发送的消息

        basic_publish 如果 exchange 不是"" 但没有绑定队列 消息会消失
        :return:
        """
        self.channel.basic_publish(exchange=exchange,
                                   routing_key=routing_key,
                                   body=body,
                                   properties=properties)

    def basic_ack(self, ch, method):
        """
        callback的消息确认
        :param ch:
        :param method:
        :return:
        """
        ch.basic_ack(delivery_tag=method.delivery_tag)

    def callback(self, ch, method, properties, body):
        """
        从队列接收消息要更复杂一些，它需要为队列订阅一个 callback 函数来进行接收。
        当我们接收一个消息后，这个 callback 函数将会被 pika函数库自动调用，
        在我们的这个实例里面这个函数将用来打印接收的消息内容到屏幕
        :param method:
        :param properties:
        :param body:
        :return:
        """
        print(type(body))
        print(" [x] Received %r" % body)
        if self.auto_ack is False:
            self.basic_ack(ch, method)

    def set_get_msg_callback(self, routing_key="hello", callback=None, auto_ack=True):
        """
        设置取消息的callback
        no_ack 如果设置为True，将使用自动确认模式
        no_ack 如果设置为False，在callback中确认
        :return:
        """
        self.auto_ack = auto_ack
        if callback is None:
            callback = self.callback
        self.channel.basic_consume(routing_key,
                                   callback,
                                   auto_ack=auto_ack)

    def start_get_msg(self):
        """
        开始取消息,会循环不停的取消息
        :return:
        """
        self.channel.start_consuming()

    def basic_qos(self, prefetch_count=1):
        """
        可以提前发送几个消息来，当auto_ack=True时无效
        prefetch_count==1 消息未处理完前不要发送信息的消息
        :return:
        """
        self.channel.basic_qos(prefetch_count=prefetch_count)

    def exchange_declare(self, exchangename="logs", type="fanout"):
        """
        fanout exchange非常简单，你从这个名字中就能猜出来，它将从Producer方收到的消息广播给所有他知道的receiver方。而这正是我们的logger记录所需要的消息。
        交换的类型
        直接交换（direct exchange）的路由算法很简单 --  消息发送到绑定键值(binding key) 刚好完全符合路由键值( routing key) 的消息队列中。

        消息发送到一个 topic交换不能是一个任意的 routing_key -- 它必须是一个用小数点 分割的单词列表。 这个字符可以是任何单词，但是通常是指定一些连接特定消息的功能。一些有效的路由键(routing key）比如：“stock.usd.nyse",
        topic 是 直接交换的升级版

        headers Exchange ：headers交换器允许你匹配AMQP消息的header而非路由键。除此之外，headers交换器和direct交换器完全一致，但性能会差很多。因此它并不太实用，而且几乎再也用不到了。
        exchangename接下来会与队列绑定
         direct  ,  topic ,  headers  和 fanout
        :return:
        """
        return self.channel.exchange_declare(exchange=exchangename,
                                             exchange_type=type)

    def queue_bind(self, exchange="logs", queue="", routing_key=""):
        """
        queue 临时队列获取 self.random_queue_declare().method.queue
        :param exchange:
        :param queue:
        :return:
        """
        self.channel.queue_bind(exchange=exchange,
                                queue=queue,
                                routing_key=routing_key)



# -*- coding: utf-8 -*-
# @Time    : 2020/11/27 16:38
# @Author  : suhong
# @File    : smb_test.py
# @Software: PyCharm
#!/usr/local/bin/python3


import fs.smbfs
from re_common.baselibrary.utils.basedir import BaseDir


class BaseSmb(object):

    def __init__(self, host=None, username=None, password=None):
        self.conn = None
        self.host = host
        self.username = username
        self.password = password

    def set_default(self):
        """
        调用该函数默认链接177
        :return:
        """
        self.host = "192.168.31.177"
        self.username = "administrator"
        self.password = "vip-812"

    def connect(self):
        """
        设置用户名 密码 进行连接
        :return:
        """
        self.conn = fs.smbfs.SMBFS(host=self.host, username=self.username, passwd=self.password)


    def close(self):
        # 关闭链接
        self.conn.close()

    def get_gen_dir(self):
        """
        获取根目录下的文件名
        :return:
        """
        for g in self.conn.listdir(""):
            yield g

    def is_exists(self, pathdir):
        """
        目录是否存在
        :param sPth:
        :return:
        """
        return self.conn.exists(pathdir)

    def is_dir(self, pathdir):
        """
        是否是目录
        :param sPth:
        :return:
        """
        return self.conn.isdir(pathdir)

    def is_file(self, pathfile):
        """
        是否是文件
        :param sPth:
        :return:
        """
        return self.conn.isfile(pathfile)

    def create_dir(self, pathdir):
        """
        创建目录 该方法可以创建多级目录
        :param pathdir:
        :return:
        """
        if not self.is_dir_exists(pathdir):
            self.conn.makedirs(pathdir)

    def get_file_size(self,path):
        """
        获取文件大小
        :param path:
        :return:
        """
        size = 0
        size +=self.conn.getsize(path)
        return size

    def download_file(self,srcpath,dstpath):
        """
        下载smb服务器文件到本地
        :param srcpath:本地文件
        :param dstpath: 服务器文件
        :return:
        """
        BaseDir.create_dir(srcpath)
        with open(dstpath, 'wb') as write_file:
            self.conn.download(srcpath,write_file)

    def upload_file(self,srcpath,dstpath):
        """
        上传本地文件到smb服务器文件
        :param srcpath:本地文件
        :param dstpath: 服务器文件
        :return:
        """
        self.create_dir(dstpath)
        with open(srcpath, 'rb') as read_file:
            self.conn.upload(dstpath,read_file)


    def copy_file_to_file_smb(self,filePath, tarPath,overwrite=False):
        """
        该方法只适合smb服务器地址到smb服务器地址
        :param filePath:  文件路径
        :param tarDirPath: 输出文件路径
        :return:
        """
        assert self.is_exists(filePath), FileNotFoundError("需要copy的文件不存在")
        assert self.conn.isfile(filePath), FileNotFoundError("需要copy的不是一个文件")

        self.conn.copy(filePath,tarPath,overwrite=overwrite)

    def copy_dir_to_dir_smb(self, oldDir, newDir, moudle=0):
        """
        该方法只适合smb服务器地址到smb服务器地址
        olddir和newdir都只能是目录，且newdir必须不存在
        :param oldDir:
        :param newDir:
        :return:
        """
        assert self.is_exists(oldDir), IsADirectoryError(oldDir + "目录不存在")
        assert not self.is_exists(newDir), IsADirectoryError(newDir + "目录存在")
        self.conn.copydir(oldDir, newDir)

    def move_file_to_file_smb(self,src_path, dst_path, overwrite=False):

        assert self.is_exists(src_path), FileNotFoundError("需要move的文件不存在")
        assert self.is_file(dst_path), FileNotFoundError("需要move的不是一个文件")
        self.conn.move(src_path, dst_path, overwrite=overwrite)

    def move_dir_to_dir_smb(self, oldDir, newDir, moudle=0):
        """
        该方法只适合smb服务器地址到smb服务器地址
        olddir和newdir都只能是目录，且newdir必须不存在
        :param oldDir:
        :param newDir:
        :return:
        """
        assert self.is_exists(oldDir), IsADirectoryError(oldDir + "目录不存在")
        assert not self.is_exists(newDir), IsADirectoryError(newDir + "目录存在")
        self.conn.movedir(oldDir, newDir)


    def delete_dir(self, dirpath):
        """
        递归删除smb服务器文件夹
        :param dirpath:
        :return:
        """
        assert self.is_file(dirpath), FileExistsError("该目录是个文件")

        self.conn.removetree(dirpath)

    def delete_file(self,filepath):
        """
       删除smb服务器文件
       :param dirpath:
       :return:
       """
        assert not self.is_file(filepath), FileNotFoundError("该路径不是个文件")

        self.conn.remove(filepath)







# if __name__ == '__main__':
    # bs = BaseSmb()
    # bs.set_default()
    # bs.connect()
    # print(bs.delete_dir("down_data/test.txt"))


#!/bin/env python
# -*- coding: utf-8-*-
#author: ganruoxun
#date: 2020-09-25

import mmh3
import os
import binascii

class Mmh3Hash(object):
    def __init__(self,_type,resType):
        self.typeDic = {
            "2":"bs",
            "3": "hy",
            "4": "zl",
            "5": "bz",
            "10": "fg",
        }
        self.typeCode = self.typeDic[_type]
        self.resType = resType
        if self.typeCode == None:
            raise RuntimeError('type 参数无法识别！')

    ##从filePath获取文件名，将文件名转大写，后缀转小写
    def normFileName(self,filePath):
        lngid=os.path.splitext(os.path.normpath(os.path.basename(filePath)))[0].upper()
        prfx=os.path.splitext(os.path.normpath(os.path.basename(filePath)))[1].lower()
        return "{0}{1}".format(lngid,prfx)

    ##规范文件名称，文件名转大写，文件后缀转小写
    def normFileName(self,fileName):
        if not "." in fileName:
            return fileName.upper()
        tmps = fileName.split(".")
        filePrfx = tmps[0].upper()
        fileSufx = tmps[1].lower()
        return "{0}{1}".format(filePrfx, fileSufx)

    ## 使用murmurhash3算法将新全文文件名进行HASH并按照规则组装成HASH目录
    def generateHashName(self,fileName):
        hashCode = binascii.b2a_hex(mmh3.hash_bytes(fileName)).upper()[0:3]
        firstCode = chr(hashCode[0])
        secondCode = chr(hashCode[1])
        thirdCode = chr(hashCode[2])
        if thirdCode.isdigit():
            return firstCode + secondCode + str(int(thirdCode)%5)
        elif thirdCode == 'D':
            return firstCode + secondCode + 'A'
        elif thirdCode == 'E':
            return firstCode + secondCode + 'B'
        elif thirdCode == 'F':
            return firstCode + secondCode + 'C'
        else:
            return firstCode + secondCode + thirdCode

    #fileName：文件名称，带后缀，不能为空，专利为公开号加文件后缀名，其他为lngid加文件后缀名
    #years：年份，不能为空
    #country：国家，如果为空，默认为cn
    #type：自建资源类型，不能为空，目前只有bs（博硕）,hy（会议）,bz（标准）,fg（法规）,zl（专利）
    def generatehashPath(self,fileName,years,country,resType):
        if  years == None or len(years) != 4:
            raise RuntimeError('years 参数错误！')
        elif fileName == None or len(fileName) == 0:
            raise RuntimeError('fileName 参数错误！')
        elif country == None or len(country) == 0:
            country = 'cn'
        if resType in ('bs', 'hy', 'fg', 'zl', 'bz'):
            country = "cn"
        country = country.lower()
        intYear = int(years)
        if intYear < 1989:
            years = 'befor1989'
        fileName = normFileName(fileName)
        return "\\" + resType + "\\" + years + country + self.typeCode + "\\" + generateFullPathName(fileName);
        return '\\' + years + country + _type + '\\' + generateHashName(fileName) + '\\' + fileName

